package com.halomobi.network.net;

import static com.halomobi.common.ConstantPool.EroType.ERROR_FILE_NOT_FOUND;
import static com.halomobi.common.ConstantPool.EroType.ERROR_UNKNOWN_HOST;
import static com.halomobi.common.ConstantPool.EroType.EXCEPTION;
import static com.halomobi.common.ConstantPool.UA_KEY;

import android.text.TextUtils;

import com.halomobi.network.net.http.HttpResult;
import com.halomobi.network.net.http.MyX509TrustManager;
import com.halomobi.network.net.http.TrustAnyHostnameVerifier;
import com.halomobi.network.request.host.bean.DnsBean;
import com.halomobi.network.schedule.ITask;
import com.halomobi.utils.FileUtils;
import com.halomobi.utils.LogUtils;
import com.halomobi.utils.NetworkUtils;
import com.halomobi.utils.Utils;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.SocketTimeoutException;
import java.net.URL;
import java.net.UnknownHostException;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;

import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;


/**
 * 网络加载抽象任务类
 *
 * @author Jeve
 * @since 2015-12-10
 */
public abstract class AbsNetTask implements ITask {

    private ReqType reqType;
    private NetTaskListener mListener;

    protected DnsBean mDnsBean;

    private boolean isGzipRequest = false;

    /**
     * HTTP链接超时时间
     */
    private static final int HTTP_TIMEOUT = 100 * 1000;
    /**
     * 请求获取数据超时时间
     */
    private static final int SOCKET_TIMEOUT = 100 * 1000;


    /**
     * 是否通过IP对应关系重发网络请求
     */
    private boolean isReTry = false;

    /**
     * 设置是否可以通过IP对应关系进行重发网络请求
     *
     * @param reTry 是否重发
     */
    protected void setReTry(boolean reTry) {
        isReTry = reTry;
    }

    public AbsNetTask(ReqType type) {
        this.reqType = type;
    }

    @Override
    public void run() {
        if (!NetworkUtils.isNetworkConnected(Utils.getContext())) {
            LogUtils.e("没有网络连接，网络任务中断");
            if (mListener != null) {
                mListener.onNetworkError();
            }
            return;
        }
        execute();
    }

    protected abstract byte[] getEntity();

    protected abstract String getUrl() throws Exception;

    public void setListener(NetTaskListener netTaskListener) {
        this.mListener = netTaskListener;
    }

    protected abstract void onAddHeaders(HttpURLConnection urlConnection);

    private void execute() {
        HttpResult httpResult = null;
        try {
            String url = getUrl();
            HttpURLConnection connection = openConnection(url);

            httpResult = getHttpResult(connection, 1, 1);

            if (httpResult == null) {
                LogUtils.e("  httpResult  == null");
                LogUtils.e("  url  " + getUrl());
                return;
            }

            int resultCode = httpResult.getCode();

            LogUtils.d("resultCode  : " + resultCode + " url :  " + url);
            if (resultCode < 200 || resultCode >= 400) { // 大于400 网络错误
                onErrorReceived("NetException:" + HttpResult.getMsgByErrorCode(resultCode), resultCode);
                return;
            }
            if (resultCode == 204) { // 大于400 网络错误
                onErrorReceived("NetException:" + HttpResult.getMsgByErrorCode(resultCode), resultCode);
                return;
            }

            if (mListener != null) {
                mListener.onDataReceived(httpResult);
            }

            //检测是否有未发送成功的错误信息
//            ReportErrorManager.getInstance().reportHisEros();
            //上报应用安装失败集合
//            ReportErrorManager.getInstance().reportInstallFailedErrors();
            //上报异常崩溃信息
//            ReportExceptionManager.getInstance().reportException();

        } catch (UnknownHostException e) {
            onErrorReceived(LogUtils.getThrowable(e), ERROR_UNKNOWN_HOST);
            e.printStackTrace();
        } catch (FileNotFoundException e) {
            onErrorReceived(LogUtils.getThrowable(e), ERROR_FILE_NOT_FOUND);
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
            onErrorReceived(LogUtils.getThrowable(e), EXCEPTION);
        } finally {
            if (httpResult != null) {
                httpResult.close();
            }
        }
    }

    private HttpResult getHttpResult(HttpURLConnection connection, int RedirectsNum, int reTryNum) throws Exception {
        if (connection == null) {
            LogUtils.e("connection == null");
            if (mListener != null) {
                mListener.onNetworkError();
            }
            return null;
        }

        if (RedirectsNum >= 5) {
            LogUtils.e("循环重定向超过5次");
            if (mListener != null) {
                mListener.onErrorReceived("循环重定向超过5次： " + getUrl(), EXCEPTION);
            }
            return null;
        }

        HttpResult httpResult;
        try {
            httpResult = new HttpResult(connection);
            int resultCode = httpResult.getCode();
            // 广告请求成功
            if (resultCode > 300 && resultCode < 400) {
                String location = connection.getHeaderField("Location");
                if (!TextUtils.isEmpty(location)) {
                    LogUtils.e("重定向 ： " + location);
                    httpResult = getHttpResult(openConnection(location), ++RedirectsNum, reTryNum);
                } else {
                    LogUtils.e(" location  == null");
                }
            }
        } catch (SocketTimeoutException e) {
            if (reTryNum >= 3) {
                LogUtils.e("已达到重试上限");
                throw e;
            }
            httpResult = getHttpResult(openConnection(getUrl()), RedirectsNum, ++reTryNum);
            e.printStackTrace();
        }
        return httpResult;
    }

    protected void setGzipRequest(boolean isGzipRequest) {
        this.isGzipRequest = isGzipRequest;
    }

    public HttpURLConnection openConnection(String url) throws Exception {

        if (TextUtils.isEmpty(url)) {
            LogUtils.e("url == null");
            return null;
        }

        URL Url = new URL(url);

        String str_http = url.substring(0, url.indexOf(":"));

        HttpURLConnection connection;
        if (str_http.equals("https")) {
            connection = getHttpsURLConnection(Url);
        } else {
            connection = getHttpURLConnection(Url);
        }
        return dealWithHttpConnection(connection);
    }

    private HttpURLConnection dealWithHttpConnection(HttpURLConnection connection) throws Exception {
        if (isGzipRequest) {
            connection.addRequestProperty("Content-Type", "text/html");
            connection.addRequestProperty("Accept-Encoding", "gzip,deflate");
        } else {
            connection.addRequestProperty("Accept-Encoding", "");
            connection.setRequestProperty("Content-Type", "application/json;charset=utf-8");
        }
        String ua = Utils.getSp().getString(UA_KEY, "");
        if (!TextUtils.isEmpty(ua)) {
//            Log.e("11111","====userAgent11111====" + ua);
            connection.addRequestProperty("User-agent", ua);
        }
        onAddHeaders(connection);
        switch (reqType) {
            case Get:
                connection.setRequestMethod("GET");
                break;
            case Post:
                connection.setRequestMethod("POST");
                connection.setDoOutput(true);
                connection.setDoInput(true);
//                //指定流的大小，当内容达到这个值的时候就把流输出
//                connection.setChunkedStreamingMode(1024 * 1024);
                OutputStream outputStream = null;
                try {
                    outputStream = connection.getOutputStream();
//                    byte[] entity = SystemUtils.gZip(getEntity());
                    byte[] entity = getEntity();
                    if (entity != null && entity.length > 0) {
                        outputStream.write(entity);
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                } finally {
                    FileUtils.close(outputStream);
                }
                break;
            default:
                break;
        }
        return connection;
    }

    private HttpURLConnection getHttpURLConnection(URL url) throws IOException {
        HttpURLConnection httpURLConnection;
        httpURLConnection = (HttpURLConnection) url.openConnection();
        httpURLConnection.setConnectTimeout(SOCKET_TIMEOUT);
        httpURLConnection.setReadTimeout(HTTP_TIMEOUT);
        httpURLConnection.setInstanceFollowRedirects(true);
        httpURLConnection.setRequestProperty("Accept-Charset", "UTF-8");
        return httpURLConnection;
    }

    private HttpsURLConnection getHttpsURLConnection(URL url) throws IOException, NoSuchAlgorithmException, KeyManagementException {
        HttpsURLConnection httpsURLConnection;
        httpsURLConnection = (HttpsURLConnection) url.openConnection();
        httpsURLConnection.setConnectTimeout(SOCKET_TIMEOUT);
        httpsURLConnection.setReadTimeout(HTTP_TIMEOUT);
        TrustManager[] tm = {new MyX509TrustManager()};
        SSLContext sslContext = SSLContext.getInstance("SSL");
        sslContext.init(null, tm, null);
        System.setProperty("https.protocols", "TLSv1,TLSv1.1,TLSv1.2");
        httpsURLConnection.setSSLSocketFactory(sslContext.getSocketFactory());
        httpsURLConnection.setHostnameVerifier(new TrustAnyHostnameVerifier());
        httpsURLConnection.setInstanceFollowRedirects(true);
        httpsURLConnection.setRequestProperty("Accept-Charset", "UTF-8");
        return httpsURLConnection;
    }

    public enum ReqType {
        Get, Post
    }

    /**
     * 请求发生错误
     *
     * @param eroMsg  错误信息
     * @param eroCode 错误码
     */
    private void onErrorReceived(String eroMsg, int eroCode) {

//        if (isReTry && (eroCode == ERROR_UNKNOWN_HOST || eroCode == ERROR_FILE_NOT_FOUND || eroCode == 503)) {
//            if (reTry()) {//通过IP对应关系进行重试
//                execute();
//                return;
//            }
//        }

        if (mListener != null) {
            mListener.onErrorReceived(eroMsg, eroCode);
        }
    }

    /**
     * 更换IP重新请求
     */
//    private boolean reTry() {
//
//        if (mDnsBean.ipList == null || mDnsBean.ipList.isEmpty()) {
//            mDnsBean.host = mDnsBean.url.replaceAll("http*?://(.+?)/.+", "$1");
//            mDnsBean.ipList = RequestIpManager.getInstance().getIpList(mDnsBean.host);
//        }
//
//        if (mDnsBean.ipList == null || mDnsBean.ipList.isEmpty()) {//没有该域名的ip对应关系
//            return false;
//        }
//
//        if (mDnsBean.position >= mDnsBean.ipList.size()) {//达到最大重试次数
//            return false;
//        }
//
//        String ip = mDnsBean.ipList.get(mDnsBean.position);
//        mDnsBean.url = mDnsBean.url.replaceAll("(http*?://).+?(/.+)", "$1" + ip + "$2");
//        mDnsBean.position++;
//        return true;
//    }

}

