package com.halomobi.network.schedule;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.Executors;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.ThreadPoolExecutor.AbortPolicy;
import java.util.concurrent.TimeUnit;


public class ThreadManager {

    private static final String DEFAULT_SINGLE_POOL_NAME = "DEFAULT_SINGLE_POOL_NAME";

    private static ThreadPoolProxy mLongPool = null;

    private static ThreadPoolProxy mShortPool = null;

    private static ThreadPoolProxy mDownloadPool = null;

    private static ThreadPoolProxy mCheckViewPool = null;

    private static ThreadPoolProxy mRequestAdPool = null;

    private static Map<String, ThreadPoolProxy> mMap = new HashMap<>();
    private static final Object mSingleLock = new Object();

    /**
     * 请求广告线程
     */
    public synchronized static ThreadPoolProxy getRequestAdPool() {
        if (mRequestAdPool == null) {
            mRequestAdPool = new ThreadPoolProxy(3, 5, 5L);
        }
        return mRequestAdPool;
    }

    /**
     * 获取下载线程
     */
    public synchronized static ThreadPoolProxy getDownloadPool() {
        if (mDownloadPool == null) {
            mDownloadPool = new ThreadPoolProxy(3, 3, 5L);
        }
        return mDownloadPool;
    }

    public synchronized static ThreadPoolProxy getCheckViewPool() {
        if (mCheckViewPool == null) {
            mCheckViewPool = new ThreadPoolProxy(3, 3, 5L);
        }
        return mCheckViewPool;
    }

    /**
     * 获取一个用于执行长耗时任务的线程池，避免和短耗时任务处在同一个队列而阻塞了重要的短耗时任务，通常用来联网操作
     */
    public synchronized static ThreadPoolProxy getLongPool() {
        if (mLongPool == null) {
            mLongPool = new ThreadPoolProxy(3, 5, 5L);
        }
        return mLongPool;
    }

    /**
     * 获取一个用于执行短耗时任务的线程池，避免因为和耗时长的任务处在同一个队列而长时间得不到执行，通常用来执行本地的IO/SQL
     */
    public synchronized static ThreadPoolProxy getShortPool() {
        if (mShortPool == null) {
            mShortPool = new ThreadPoolProxy(2, 2, 5L);
        }
        return mShortPool;
    }

    /**
     * 获取一个单线程池，所有任务将会被按照加入的顺序执行，免除了同步开销的问题
     */
    public static ThreadPoolProxy getSinglePool() {
        return getSinglePool(DEFAULT_SINGLE_POOL_NAME);
    }

    /**
     * 获取一个单线程池，所有任务将会被按照加入的顺序执行，免除了同步开销的问题
     */
    public static ThreadPoolProxy getSinglePool(String name) {
        synchronized (mSingleLock) {
            ThreadPoolProxy singlePool = mMap.get(name);
            if (singlePool == null) {
                singlePool = new ThreadPoolProxy(1, 1, 1L);
                mMap.put(name, singlePool);
            }
            return singlePool;
        }
    }

    public static void onDestroy() {
        Set<Map.Entry<String, ThreadPoolProxy>> entries = mMap.entrySet();
        for (Map.Entry<String, ThreadPoolProxy> entry : entries) {
            entry.getValue().shutdown();
        }
        mMap.clear();
        getCheckViewPool().stop();
    }

    public static class ThreadPoolProxy {
        private ThreadPoolExecutor mPool;
        private int mCorePoolSize;
        private int mMaximumPoolSize;
        private long mKeepAliveTime;

        private ThreadPoolProxy(int corePoolSize, int maximumPoolSize, long keepAliveTime) {
            mCorePoolSize = corePoolSize;
            mMaximumPoolSize = maximumPoolSize;
            mKeepAliveTime = keepAliveTime;
        }

        /**
         * 执行任务，当线程池处于关闭，将会重新创建新的线程池
         */
        public synchronized void execute(Runnable task) {
            if (task == null) {
                return;
            }
            if (mPool == null || mPool.isShutdown()) {
                mPool = new ThreadPoolExecutor(mCorePoolSize, mMaximumPoolSize, mKeepAliveTime,
                        TimeUnit.SECONDS, new LinkedBlockingQueue<Runnable>(),
                        Executors.defaultThreadFactory(), new AbortPolicy());
            }
            mPool.execute(task);
        }

        /**
         * 取消线程池中某个还未执行的任务
         */
        public synchronized boolean cancel(Runnable run) {
            return mPool != null && (!mPool.isShutdown() || mPool.isTerminating()) && mPool.getQueue().remove(run);
        }

        /**
         * 判断队列中是否包含该任务
         */
        public synchronized boolean contains(Runnable run) {
            return mPool != null && (!mPool.isShutdown() || mPool.isTerminating()) && mPool.getQueue().contains(run);
        }

        /**
         * 立刻关闭线程池，并且正在执行的任务也将会被中断
         */
        synchronized void stop() {
            if (mPool != null && (!mPool.isShutdown() || mPool.isTerminating())) {
                mPool.shutdownNow();
            }
        }

        /**
         * 平缓关闭单任务线程池，但是会确保所有已经加入的任务都将会被执行完毕才关闭
         */
        synchronized void shutdown() {
            if (mPool != null && (!mPool.isShutdown() || mPool.isTerminating())) {
                mPool.shutdown();
            }
        }
    }

}
